"""Top-level package for ePages provisioning."""
import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())
# from .provisioning import *

__author__ = """Tatu Wikman"""
__email__ = 'tatu.wikman@gmail.com'
__version__ = '0.2.0'
